/**********************************************************************
 *
 * comtypes.h -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: comtypes.h 199 1999-03-11 00:11:19Z rjmcnab $
 *
 *********************************************************************/


#ifndef COMTYPES_H
#define COMTYPES_H

#include "gsdlconf.h"
#include "text_t.h"

#if defined(GSDL_USE_OBJECTSPACE)
#  include <ospace\std\vector>
#  include <ospace\std\list>
#  include <ospace\std\map>
#elif defined(GSDL_USE_STL_H)
#  include <vector.h>
#  include <list.h>
#  include <map.h>
#else
#  include <vector>
#  include <list>
#  include <map>
#endif


enum comerror_t {noError, authenticationFailure, protocolError, systemProblem};
text_t get_comerror_string (comerror_t err);

// ShortColInfo ::= SEQUENCE {
//   name GeneralString,
//   host GeneralString,
//   port INTEGER
// }
struct ShortColInfo_t {
  void clear ();
  ShortColInfo_t () {clear();}

  text_t name;
  text_t host;
  int port;
};


// ColInfoResponse ::= SEQUENCE {
//   shortInfo   [0] IMPLICIT ShortCollectionInfo,
//   isPublic    [2] IMPLICIT BOOLEAN,    -- whether has anonymous access
//   isBeta      [3] IMPLICIT BOOLEAN,    -- beta if still under development
//   buildDate   [4] IMPLICIT GeneralizedTime,
//   languages   [5] IMPLICIT StringSet,  -- languages in the collection
//   numDocs     [6] IMPLICIT INTEGER,
//   numWords    [7] IMPLICIT INTEGER OPTIONAL,
//   numBytes    [8] IMPLICIT INTEGER OPTIONAL
// }
struct ColInfoResponse_t {
  void clear ();
  ColInfoResponse_t () {clear();}

  ShortColInfo_t shortInfo;
  bool isPublic;
  bool isBeta;
  unsigned long buildDate;
  text_tarray languages;
  unsigned long numDocs;  // 0 if not known
  unsigned long numWords; // 0 if not known
  unsigned long numBytes; // 0 if not known
};



// FilterOption ::= SEQUENCE {
//   name         GeneralString,
//   type         ENUMERATED {booleant(0), integert(1), enumeratedt(2), stringt(3)},
//   repeatable   ENUMERATED {onePerQuery(0), onePerTerm(1), nPerTerm(2)},
//   defaultValue GeneralString,
//   -- the interpretation of the validValues depends on the type
//   -- for boolean: the first value is the false value, the second is true value
//   -- for integer: the first value is the minimum, the second the maximum
//   -- for enumerated: all values a listed
//   -- for string: this value is ignored
//   validValues  StringSequence
// }
struct FilterOption_t {
  void clear ();
  void check_defaultValue ();
  FilterOption_t () {clear();}

  text_t name;

  enum type_t {booleant=0, integert=1, enumeratedt=2, stringt=3};
  type_t type;

  enum repeatable_t {onePerQuery=0, onePerTerm=1, nPerTerm=2};
  repeatable_t repeatable;

  text_t defaultValue;
  text_tarray validValues;
};

typedef map<text_t, FilterOption_t, lttext_t> FilterOption_tmap;

// FilterDescript ::= SEQUENCE {
//   filterName GeneralString,
//   filterOptions SET OF FilterOption
// }
struct FilterDescript_t {
  void clear ();

  text_t filterName;
  FilterOption_tmap filterOptions;
};

typedef map<text_t, FilterDescript_t, lttext_t> FilterDescript_tmap;

// InfoFilterOptionsResponse ::= SEQUENCE {
//   filterOptions SET OF FilterDescript
// }
struct InfoFilterOptionsResponse_t {
  void clear ();

  FilterDescript_tmap filterOptions;
};



// OptionValue ::= SEQUENCE {
//   name  GeneralString,
//   value GeneralString
// }
struct OptionValue_t {
  void clear ();

  text_t name;
  text_t value;
};

typedef vector<OptionValue_t> OptionValue_tarray;


// -- Terms are presented in the same order that they are requested,
// -- any information relating to the terms is in reference to the
// -- index specified for that term.
// -- Metadata is presented in the same order that it is requested.
// 
// FilterRequest ::= SEQUENCE {
//   filterName          [0] GeneralString,
//   filterOptions       [1] IMPLICIT SEQUENCE OF OptionValue,
//   docSet              [2] IMPLICIT StringSequence OPTIONAL,
//   filterResultOptions [3] BIT STRING {termFreq(0), matchTerms(1), OID(2), subCol(3),
//                                       ranking(4), docFreq(5)}
// }
#define FRtermFreq   1
#define FRmatchTerms 2
#define FROID        4
#define FRsubCol     8
#define FRranking    16
#define FRdocFreq    32

struct FilterRequest_t {
  void clear ();
  FilterRequest_t () {clear();}

  text_t filterName;
  OptionValue_tarray filterOptions;
  text_tarray docSet;  // empty if not used
  int filterResultOptions; // use the FR* defines above
};


// TermInfo ::= SEQUENCE {
//   term       [0] GeneralString,
//   freq       [0] IMPLICIT INTEGER OPTIONAL,
//   matchTerms [1] IMPLICIT StringSequence OPTIONAL
// }
struct TermInfo_t {
  void clear ();
  TermInfo_t () {clear();}

  text_t term;
  int freq; // 0 if not requested
  text_tarray matchTerms; // empty if not requested
};

typedef vector<TermInfo_t> TermInfo_tarray;

// ResultDocInfo ::= SEQUENCE {
//   OID      [0] IMPLICIT GeneralString,
//   subCol   [1] IMPLICIT StringSequence OPTIONAL,
//   ranking  [2] IMPLICIT REAL OPTIONAL,
//   docFreq  [3] IMPLICIT SEQUENCE OF INTEGER OPTIONAL,
// }
struct ResultDocInfo_t {
  void clear ();
  ResultDocInfo_t () {clear();}

  text_t OID;
  text_tarray subCol; // empty if not requested
  float ranking; // 0.0 if not requested
  vector<int> docFreq; // empty if not requested

  ResultDocInfo_t &operator=(const ResultDocInfo_t &x);
};

typedef vector<ResultDocInfo_t> ResultDocInfo_tarray;

// FilterResponse ::= SEQUENCE {
//   docsFound [0] IMPLICIT INTEGER,
//   isApprox  [1] IMPLICIT BOOLEAN, -- whether docsFound is approximate
//   termInfo  [2] IMPLICIT SEQUENCE OF TermInfo OPTIONAL,
//   docInfo   [3] IMPLICIT SEQUENCE OF ResultDocInfo OPTIONAL
// }
struct FilterResponse_t {
  void clear ();
  FilterResponse_t () {clear();}

  int docsFound;
  bool isApprox;
  TermInfo_tarray termInfo; // empty if not requested
  ResultDocInfo_tarray docInfo; // empty if not requested
};



// MetadataInfoResponse ::= SEQUENCE {
//   supportedFields    StringSet,
//   supportedLanguages StringSet
// }
struct MetadataInfoResponse_t {
  void clear ();

  text_tset supportedFields;
  text_tset supportedLanguages;
};

// MetadataRequest ::= SEQUENCE {
//   OIDs      StringSequence,
//   fields    StringSequence,
//   languages StringSequence
// }
struct MetadataRequest_t {
  void clear ();

  text_tarray OIDs;
  text_tarray fields;
  text_tarray languages;
};


// -- for each OID it will return all the fields
// -- in each of the languages
// MetadataResponse ::= SEQUENCE {
//   metadata SEQUENCE OF GeneralString
// }
struct MetadataResponse_t {
  void clear ();

  text_tarray metadata;
};


#endif
